@@ -22,8 +22,25 @@ CHARSET target_charset;
 /* Converts char in input charset into unicode representation           */
 /* Should be converted to macro                                         */
 /************************************************************************/
+#define MBCS_MAX 1024
+struct mbcs_map {
+	int mbcs;
+	unsigned short unicode;
+};
+
+static struct mbcs_map *mbcs_charset = NULL;
+static int mbcs_count;
 int to_unicode (short int *charset, int c) {
-	return charset[c];
+	if ((c > 0) && (c <= 255))
+		return charset[c];
+	else {
+		int i;
+		for (i = 0; i < mbcs_count; i++) {
+			if (mbcs_charset[i].mbcs == c)
+				return mbcs_charset[i].unicode;
+		}
+	}
+	return -1;
 }
 /************************************************************************/
 /* Search inverse charset record for given unicode char and returns     */
@@ -68,11 +85,15 @@ CHARSET make_reverse_map(short int *charset) {
 /************************************************************************/
 /* Reads charset file (as got from ftp.unicode.org) and returns array of*/
 /* 256 short ints (malloced) mapping from charset t unicode             */
+/* additional MBCS_MAX for mbcs mapping from charset to unicode         */
 /************************************************************************/
 short int * read_charset(const char *filename) {
 	char *path;
 	FILE *f;
+	int reset_mcbs = 1;
 	short int *new=calloc(sizeof(short int),256);
+	if (!mbcs_charset)
+		mbcs_charset=calloc(sizeof(struct mbcs_map),MBCS_MAX);
 	int c;
 	long int uc;
 	path= find_file(stradd(filename,CHARSET_EXT),charset_path);
@@ -83,7 +104,7 @@ short int * read_charset(const char *filename) {
 	f=fopen(path,"rb");
 
 	if (!f) {
-		perror(path); 
+		perror(path);
 		return NULL;
 	}
 	if (input_buffer)
@@ -94,12 +115,27 @@ short int * read_charset(const char *filename) {
 	}
 	while (!feof(f)) {
 		if (fscanf(f,"%i %li",&c,&uc)==2) {
-			if (c<0||c>255||uc<0||(uc>0xFEFE&& uc!=0xFFFE)) {
+			// store first MBCS_MAX mbcs chars
+			if ((c >= 0x8000) && (c <= 0xFFFF)) {
+				if (reset_mcbs) {
+					reset_mcbs = 0;
+					mbcs_count = 0;
+				}
+				if (mbcs_count < MBCS_MAX) {
+					mbcs_charset[mbcs_count].mbcs = c;
+					mbcs_charset[mbcs_count].unicode = uc;
+					mbcs_count++;
+				}
+			} else
+//			if (c<0||c>255||uc<0||(uc>0xFEFE && uc!=0xFFFE)) {
+// shiftjis has values > 0xFEFE that are not FFFE
+			if (c<0||c>255||uc<0) {
 				fprintf(stderr,"Invalid charset file %s\n",path);
 				fclose(f);
 				return NULL;
 			}
-			new[c]=uc;
+			else
+				new[c]=uc;
 		}
 		while((fgetc(f)!='\n')&&!feof(f)) ;
 	}
@@ -292,6 +328,12 @@ const char *charset_from_codepage(unsigned int codepage) {
 		/* For UCS2 */
 		return "";
 	} else 
+	if (codepage == 932)
+		return "shiftjis";
+	else
+	if (codepage == 950)
+		return "big5";
+	else
 	if (codepage<10000) {
 		sprintf(buffer,"cp%d",codepage);
 		return buffer;
