@@ -2,7 +2,7 @@
  * @file   ole.h
  * @author Alex Ott <ott@jet.msk.su>
  * @date   03  2003
- * Version: $Id: ole.h,v 1.5 2005/03/15 20:06:51 vitus Exp $
+ * Version: $Id: ole.h,v 1.1.1.1 2006/02/24 17:44:06 vitus Exp $
  * Copyright: Alex Ott, 2003, 
  * 
  * @brief  
@@ -22,7 +22,7 @@
 typedef enum {
 	oleDir=1,
 	oleStream=2,
-	oleRoorDir=5,
+	oleRootDir=5,
 	oleUnknown=3
 } oleType;
 
