@@ -589,8 +589,8 @@ static char FormatIdxUsed[NUMOFDATEFORMATS];
 
 void CleanUpFormatIdxUsed() {
 	int i;
-	for (i=0;i<NUMOFDATEFORMATS; i++);
-	FormatIdxUsed[i]=0;
+	for (i=0;i<NUMOFDATEFORMATS; i++)
+		FormatIdxUsed[i]=0;
 }
 
 /* 
